/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.portlets.content.jcr.renderer.html;

import java.io.IOException;
import java.util.ResourceBundle;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.jcr.*;
import javax.jcr.nodetype.NodeDef;
import javax.jcr.nodetype.PropertyDef;
import javax.jcr.version.OnParentVersionAction;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.content.jcr.component.JCRExplorer;

/**
 * @author <a href="tuan08@users.sourceforge.net">Tuan Nguyen</a>
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: JCRExplorerRenderer.java,v 1.1 2004/11/02 20:02:53 geaz Exp $
 */
public class JCRExplorerRenderer extends HtmlBasicRenderer {
  final public static String VIEW_ICON = "<img class='view-icon' src='/skin/blank.gif'/>";
  final public static String EDIT_ICON = "<img class='edit-icon' src='/skin/blank.gif'/>";
  final public static String DELETE_ICON = "<img class='delete-icon' src='/skin/blank.gif'/>";
  
  protected static Parameter CHANGE_NODE_PARAM = new Parameter(ACTION, JCRExplorer.CHANGE_NODE);
  protected static Parameter CHANGE_PROPERTY_PARAM = new Parameter(ACTION, JCRExplorer.CHANGE_PROPERTY);
  protected static Parameter SAVE_SESSION_PARAM = new Parameter(ACTION, JCRExplorer.SAVE_SESSION);
  /*
   protected static Parameter SHOW_EDITOR = new Parameter(ACTION, UIExplorer.EDIT_NODE_CONTENT);
   protected static Parameter UPLOAD_FILE = new Parameter(ACTION, UIExplorer.UPLOAD_FILE);
   protected static Parameter NEW_FOLDER = new Parameter(ACTION, "newFolder");
   
   protected static Parameter DELETE = new Parameter(ACTION, UIExplorer.REMOVE_NODE);
   protected static Parameter VIEW = new Parameter(ACTION, UIExplorer.VIEW_NODE_CONTENT);
   protected static Parameter EDIT = new Parameter(ACTION, UIExplorer.EDIT_NODE_CONTENT);
   */
  private JCRExplorer uiExplorer;
  
  public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
    
    uiExplorer = (JCRExplorer)component;
    ResourceBundle res = getApplicationResourceBundle(context.getExternalContext());
    
//  System.out.println("RES>>>>>>>>>"+res.getKeys().nextElement());
    
    Item currentItem = uiExplorer.getSelectedItem();
    Node currentNode = getThisOrParentNode(currentItem);
    
    ResponseWriter w = context.getResponseWriter();
    
    //render header
    w.write("<table class='JCRExplorer'>");
    w.write("<tr>");
    w.write("<td class='header' colspan='2'>");
    w.write(currentItem.getPath());
    w.write("</td>");
    w.write("</tr>");
    
    //render body
    w.write("<tr>");
    w.write("<td>");
    renderNodeTree(w, res, currentItem);
    w.write("</td>");
    w.write("<td>");
    if (currentItem.isNode())
      renderNodeDetail(w, res, currentNode);
    else
      renderPropertyDetail(w, res, (Property)currentItem);
    w.write("</td>");
    w.write("</tr>");
    
    //render footer
    Parameter parentUriParam = new Parameter("parentUri", currentItem.getPath());
    Parameter[] saveSessionParams = { SAVE_SESSION_PARAM };
    /*
     Parameter[] showUploadParams = { UPLOAD_FILE , parentUriParam };
     Parameter[] newFolderParams = { NEW_FOLDER , parentUriParam };
     */
    w.write("<tr>");
    w.write("<td class='footer' colspan='2'>");
    uiExplorer.appendLink(w, "[Save]", saveSessionParams, "");
    w.write(getNbsp(2));
    /*
     uiExplorer.appendLink(w, "[Revert]", showUploadParams, "");
     w.write(getNbsp(2));
     uiExplorer.appendLink(w, "[Import XML]", newFolderParams, "");
     */
    w.write("</td>");
    w.write("</tr>");
    w.write("</table>");
  }
  
  private void renderNodeTree(ResponseWriter w, ResourceBundle res, Item currentItem) throws IOException {
    
    NodeIterator children;
    PropertyIterator properties;
    Parameter uriParam;
    Parameter propertyPath;
    Node node = getThisOrParentNode(currentItem);
    
    try {
      children = node.getNodes();
      properties = node.getProperties();
    } catch (Exception e) {
      throw new IOException("Render Exception " + e);
    }
    
    int indent = 0;
    try {
      int depth = (currentItem.isNode()) ? currentItem.getDepth() + 1 : currentItem.getDepth();
      for (int i = 0; i < depth; i++) {
        indent += i * 2;
        if(currentItem.getPath().equals(currentItem.getAncestor(i).getPath()))
          w.write("<div class='selected'>");
        else
          w.write("<div>");
        w.write(getNbsp(indent));
        w.write("<img src='"); w.write(res.getString("img.node-opened")); w.write("'/>");
        uiExplorer.appendLink(w, "/" + currentItem.getAncestor(i).getName(), getChangeNodeParameters(currentItem.getAncestor(i)), "");
        w.write("</div>");
      }
    } catch (Exception e) {
      throw new IOException("Render Exception " + e);
    }
    
    indent += 2;
    Property property;
    
    while (properties.hasNext()) {
      property = properties.nextProperty();
      if(currentItem.getPath().equals(property.getPath()))
        w.write("<div class='selected'>");
      else
        w.write("<div>");
      w.write(getNbsp(indent));
      w.write("<img src='"); w.write(res.getString("img.property")); w.write("'/>");
      uiExplorer.appendLink(w, property.getName(), getChangePropertyParameters(property), "");
      w.write("</div>");
    }
    
    while (children.hasNext()) {
      w.write("<div>");
      w.write(getNbsp(indent));
      w.write("<img src='"); w.write(res.getString("img.node-closed")); w.write("'/>");
      Node child = children.nextNode();
      uiExplorer.appendLink(w, child.getName(), getChangeNodeParameters(child), "");
      w.write("</div>");
    }
  }
  
  private void renderNodeDetail(ResponseWriter w, ResourceBundle res, Node node) throws IOException {
    
    Parameter uriParam = new Parameter("uri", "");
    Parameter parentUriParam = new Parameter("parentUri", "");
    Parameter localNameParam = new Parameter("localName", "");
    /*
     Parameter[] deleteNodeParams = { DELETE , uriParam};
     Parameter[] editNodeParams = { EDIT , parentUriParam, localNameParam};
     Parameter[] viewNodeParams = { VIEW , uriParam};
     */
    // Attrs
    renderNodeAttributes(node, w, res);
    w.write("<br>");
    // Properies
    renderNodeProperties(node, w, res);
    w.write("<br>");
    // Definition
    renderNodeDefinition(node, w, res);
  }
  
  protected void renderPropertyDetail(ResponseWriter w, ResourceBundle res, Property property) throws IOException {
    // Attrs
    renderPropertyAttributes(property, w, res);
    w.write("<br>");
    // Values
    renderPropertyValues(property, w, res);
    w.write("<br>");
    // Definition
    renderPropertyDefinition(property, w, res);
  }
  
  private void renderNodeAttributes(Node node, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<th>"); w.write("Details"); w.write("</th>");
    w.write("<th>"); w.write(""); w.write("</th>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Name"); w.write("</td>");
    w.write("<td>"); w.write((node.getName()));
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Paths"); w.write("</td>");
    w.write("<td>");
    StringIterator paths = node.getPaths();
    while(paths.hasNext())
      w.write(paths.nextString()+" ");
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Primary Node Type"); w.write("</td>");
    w.write("<td>"); w.write((node.getPrimaryNodeType().getName()));
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Mixin Node Types"); w.write("</td>");
    w.write("<td>");
    for(int i=0;i<node.getMixinNodeTypes().length; i++)
      w.write(node.getMixinNodeTypes()[i].getName());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Primary Item"); w.write("</td>");
    w.write("<td>");
    try {
      w.write((node.getPrimaryItem().getPath()));
    } catch (ItemNotFoundException e) {
    } catch (RepositoryException e) {
      w.write(e.getMessage());
    }
    w.write("</td>");
    w.write("</tr>");
    
    w.write("</table>");
  }
  
  private void renderNodeProperties(Node node, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<th>"); w.write("Property"); w.write("</th>");
    w.write("<th>"); w.write("Value"); w.write("</th>");
    w.write("<th>"); w.write("Type"); w.write("</th>");
    PropertyIterator properties;
    try {
      properties = node.getProperties();
    } catch (RepositoryException e) {
      throw new IOException("Render Exception " + e);
    }
    int i = 0;
    Property property;
    Value value;
    while (properties.hasNext()) {
      try {
        property = properties.nextProperty();
        value = property.getValue();
      } catch (RepositoryException e) {
        throw new IOException("Render Exception " + e);
      }
      w.write("<tr>");
      w.write("<td width='30%'>"); w.write(property.getName()); w.write("</td>");
      try {
        w.write("<td>"); w.write(value.getString()); w.write("</td>");
      } catch (RepositoryException e) {
        w.write("<td>"); w.write("Format error"); w.write("</td>");
      }
      w.write("<td>"); w.write(PropertyType.nameFromValue(value.getType())); w.write("</td>");
      w.write("</tr>");
    }
    w.write("</table>");
  }
  
  private void renderNodeDefinition(Node node, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<th>"); w.write("Primary Type Definition"); w.write("</th>");
    w.write("<th>"); w.write(""); w.write("</th>");
    NodeDef nodeDef;
    try {
      nodeDef = node.getDefinition();
    } catch (Exception e) {
      throw new IOException("Render Exception " + e);
    }
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Name"); w.write("</td>");
    w.write("<td>"); w.write((nodeDef.getName() == null) ? "&lt;residual set&gt;" : nodeDef.getName());
    w.write("</td>");
    w.write("</tr>");
    
    /*
     w.write("<tr>") ;
     w.  write("<td style='text-align: left'>") ; w.write("Declaring node type") ; w.write("</td>") ;
     w.  write("<td style='text-align: left'>") ; w.write((nodeDef.getDeclaringNodeType()==null)?"Undefined":nodeDef.getDeclaringNodeType().getName()) ; w.write("</td>") ;
     w.write("</tr>") ;
     */
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Default primary type"); w.write("</td>");
    w.write("<td>"); w.write(nodeDef.getDefaultPrimaryType().getName()); w.write("</td>");
    w.write("</tr>");
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Auto create"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(nodeDef.isAutoCreate()).toString()); w.write("</td>");
    w.write("</tr>");
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Mandatory"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(nodeDef.isMandatory()).toString()); w.write("</td>");
    w.write("</tr>");
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Primary item"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(nodeDef.isPrimaryItem()).toString()); w.write("</td>");
    w.write("</tr>");
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Protected"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(nodeDef.isReadOnly()).toString()); w.write("</td>");
    w.write("</tr>");
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("On Parent Version action"); w.write("</td>");
    w.write("<td>"); w.write(OnParentVersionAction.nameFromValue(nodeDef.getOnParentVersion()));
    w.write("</td>");
    w.write("</tr>");
    w.write("</table>");
  }
  
  private void renderPropertyAttributes(Property property, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<tr>");
    w.write("<th>"); w.write("Details"); w.write("</th>");
    w.write("<th>"); w.write(""); w.write("</th>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Name"); w.write("</td>");
    w.write("<td>"); w.write(property.getName());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Paths"); w.write("</td>");
    w.write("<td>");
    StringIterator paths = property.getPaths();
    while(paths.hasNext())
      w.write(paths.nextString()+" ");
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Type"); w.write("</td>");
    w.write("<td>");
    try {
      w.write(PropertyType.nameFromValue(property.getValue().getType()));
    } catch (RepositoryException e) {
      w.write("Format error "+e.getMessage());
    }
    w.write("</td>");
    w.write("</tr>");
    
    w.write("</table>");
  }
  
  private void renderPropertyValues(Property property, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<tr>");
    w.write("<th>"); w.write("Values"); w.write("</th>");
    w.write("</tr>");
    
    Value[] values;
    try {
      values = property.getValues();
    } catch (RepositoryException e) {
      throw new IOException("Render Exception " + e);
    }
    
    for(int i=0; i<values.length; i++) {
      String clazz = "odd";
      if (i % 2 == 0)
        clazz = "even";
      w.write("<tr class='" + clazz + "'>");
      try {
        w.write("<td>"); w.write(values[i].getString()); w.write("</td>");
      } catch (Exception e) {
        e.printStackTrace();
        w.write("<td>"); w.write(e.getMessage()); w.write("</td>");
      }
      w.write("</tr>");
    }
    
    w.write("</table>");
  }
  
  private void renderPropertyDefinition(Property property, ResponseWriter w, ResourceBundle res) throws IOException {
    w.write("<table class='detail'>");
    w.write("<th>"); w.write("Primary Type Definition"); w.write("</th>");
    w.write("<th>"); w.write(""); w.write("</th>");
    PropertyDef propertyDef;
    try {
      propertyDef = property.getDefinition();
    } catch (Exception e) {
      throw new IOException("Render Exception " + e);
    }
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Name"); w.write("</td>");
    w.write("<td>"); w.write((propertyDef.getName() == null) ? "&lt;residual set&gt;" : propertyDef.getName());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Default Value"); w.write("</td>");
    
    try {
      w.write("<td>"); w.write((propertyDef.getDefaultValue() == null) ? "" : propertyDef.getDefaultValue().getString());
    } catch (RepositoryException e) {
      w.write("<td>"); w.write("Format error "+e.getMessage());
    }
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Required Type"); w.write("</td>");
    w.write("<td>");
    try {
      w.write(PropertyType.nameFromValue(propertyDef.getRequiredType()));
    } catch (IllegalArgumentException e) {
      w.write("Unknown");
    }
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Value Constraint"); w.write("</td>");
    w.write("<td>");
    String valueConstraint = propertyDef.getValueConstraint();
    if(valueConstraint != null)
      w.write(valueConstraint);
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Auto Create"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(propertyDef.isAutoCreate()).toString());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Mandatory"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(propertyDef.isMandatory()).toString());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Multiple"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(propertyDef.isMultiple()).toString());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Is Primary Item"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(propertyDef.isPrimaryItem()).toString());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("Protected"); w.write("</td>");
    w.write("<td>"); w.write(new Boolean(propertyDef.isReadOnly()).toString());
    w.write("</td>");
    w.write("</tr>");
    
    w.write("<tr>");
    w.write("<td class='detail-title'>"); w.write("On Parent Version action"); w.write("</td>");
    w.write("<td>"); w.write(OnParentVersionAction.nameFromValue(propertyDef.getOnParentVersion()));
    w.write("</td>");
    w.write("</tr>");
    
    w.write("</table>");
  }
  
  private Node getThisOrParentNode(Item item) throws IOException {
    try {
      if (item.isNode())
        return (Node)item;
      else
        return item.getParent();
    } catch (Exception e) {
      throw new IOException("Render Exception " + e);
    }
  }
  
  private Parameter[] getChangeNodeParameters(Item item) throws IOException {
    Parameter uriParam = new Parameter("uri", item.getPath());
    Parameter[] changeNodeParams = new Parameter[2];
    changeNodeParams[0] = CHANGE_NODE_PARAM;
    changeNodeParams[1] = uriParam;
    return changeNodeParams;
  }
  
  private Parameter[] getChangePropertyParameters(Item item) throws IOException {
    Parameter uriParam = new Parameter("uri", item.getPath());
    Parameter[] changePropertyParams = new Parameter[2];
    changePropertyParams[0] = CHANGE_PROPERTY_PARAM;
    changePropertyParams[1] = uriParam;
    return changePropertyParams;
  }
  
  private String getNbsp(int time) {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < time; i++)
      buf.append("&nbsp;");
    return new String(buf);
  }
}
