/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.communication.forum.hibernate;

import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import com.thoughtworks.xstream.XStream;
import net.sf.hibernate.Hibernate;
import net.sf.hibernate.Query;
import net.sf.hibernate.Session;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 21, 2004
 * @version $Id$
 */
public class ForumExporter {
  private Session session_ ;
  private JarOutputStream out_ ;
  private XStream xstream_ ;
  
  public ForumExporter(Session session, JarOutputStream out, XStream xstream) {
    session_ = session ;
    out_ = out ;
    xstream_ = xstream ;
  }
  
  public void export() throws Exception {
    Query q = session_.createQuery(ForumServiceContainerImpl.queryForumOwners);
    List owners = q.list() ;
    for(int i = 0; i < owners.size(); i++ ) {
      String owner = (String) owners.get(i) ;
      export(owner) ;
    }
  }
  
  public void export(String owner) throws Exception {
    List  categories = session_.find(ForumServiceImpl.queryCategoriesByOwner, owner, Hibernate.STRING );
    for(int i = 0; i < categories.size(); i++) {
      CategoryImpl cat = (CategoryImpl) categories.get(i) ;
      exportCategory(owner, cat) ; 
    }
  }
  
  public void exportCategory(String path ,CategoryImpl category) throws Exception {
    String categoryPath = path + "/"  + category.getId() ;
    JarEntry entry = new JarEntry(categoryPath + "/" + category.getId() +  ".category") ;
    out_.putNextEntry(entry) ;
    String xml = xstream_.toXML(category) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
    List forums = session_.find(ForumServiceImpl.queryForumsByCategory, category.getId(), Hibernate.STRING );
    for(int i = 0; i < forums.size(); i++) {
      ForumImpl forum = (ForumImpl)  forums.get(i) ;
      exportForum(categoryPath ,forum) ;
    }
  }
  
  public void exportForum(String path ,ForumImpl forum) throws Exception {
    String forumPath  =  path + "/"  +forum.getId(); 
    JarEntry entry = new JarEntry(forumPath+ "/" + forum.getId() +  ".forum") ;
    out_.putNextEntry(entry) ;
    String xml = xstream_.toXML(forum) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
    String queryTopicsByForum =
      "from topic in class org.exoplatform.services.communication.forum.hibernate.TopicImpl " +
      "where topic.forumId = ?";
    List topics = session_.find(queryTopicsByForum, forum.getId(), Hibernate.STRING );
    for(int i = 0; i < topics.size();  i++) {
      TopicImpl topic = (TopicImpl) topics.get(i) ;
      exportTopic(forumPath, topic) ;
    }
  }
  
  public void exportTopic(String path ,TopicImpl topic) throws Exception {
    JarEntry entry = new JarEntry(path + "/" + topic.getId() +  ".topic") ;
    out_.putNextEntry(entry) ;
    List  posts = session_.find(ForumServiceImpl.queryPostsByTopic, topic.getId(), Hibernate.STRING) ;
    TopicBackup backup = new TopicBackup(topic, posts) ;
    String xml = xstream_.toXML(backup) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
  }
}
